#include "main.h"
#include <stdlib.h>

//#define DEBUG

//#define CLOCKSPEED_32MHZ
#define CLOCKSPEED_16MHZ
//#define CLOCKSPEED_8MHZ

// >>>>>> Prototypen <<<<<

byte Display_CheckForSlave();
void Init_SPI(byte);
byte LCD_Clear();

/////////////////////////////////////////////////////////////////////////////////
//						globaler Variablenteil
/////////////////////////////////////////////////////////////////////////////////

// >>> Informations-Flags ber Systemzustand (Read Only) <<<<<

bit	f_IRRecBufFull;			// R   1 = Empfangs Buffer voll, 0 = Buffer leer (wird von IRQ gesetzt)

// >>> Diverse Buffer <<<<
byte LCD[25];			// Buffer fr Zeichenkette auf Display
byte IRTransBuf[80];		// IR Buffer fr Sendung
byte IRRecBuf[80];			// IR Buffer fr Empfang

int16 Errorcounter;
int16 Datensaetze;

// ---------------------------------------------------------------------------------------------

// Include Files

#include "..\definitions.h"
#include "systemdrive.c"
#include "rfm12.c"
#include "ds1306.c"
#include "iredcom.c"
#include "inter.c"
#include "display.c"
#include "datecalc.c"


// ---------------------------------------------------------------------------------------------

// Klingelt mal berall kurz an
void TestDevices()
{
	// Teste LCD Controller....
	LCD_Clear();
	
	LCD_Write(LCD,1);
	if (Test_SPI_Device(devLCD))
		sprintf(LCD,"  OK");
	else
		sprintf(LCD,"FAIL");	

	LCD_Write(LCD,0);

	delay_ms(300);

	sprintf(LCD,"IR Interface....");
	LCD_Write(LCD,2);
	if (Test_SPI_Device(devIR))
		sprintf(LCD,"  OK");
	else
		sprintf(LCD,"FAIL");	

	LCD_Write(LCD,0);

	delay_ms(1000);
}	

// --------------------------------------------------------------------	

void InitSystem()
{

	restart_wdt();

	#ifdef CLOCKSPEED_32MHZ	
		#use delay (clock=32000000,restart_wdt)
		setup_oscillator(OSC_32MHZ | OSC_PLL_ON);
	#endif	
	#ifdef CLOCKSPEED_16MHZ	
		#use delay (clock=16000000,restart_wdt)
		setup_oscillator(OSC_16MHZ | OSC_PLL_ON);
	#endif	
	#ifdef CLOCKSPEED_8MHZ	
		#use delay (clock=8000000,restart_wdt)
		setup_oscillator(OSC_8MHZ);
	#endif	

	OSCTUNE = OSCTUNE | 0b00000001;

	// Einstellungen fr AD Wandler

	// >>>>  Ports lschen  <<<<

	TRISA = 0b11111111;
	TRISB = 0b11011111;
	TRISC = 0b10010100;
	TRISD = 0b11011000;
	TRISE = 0b00000000;
	
	port_b_pullups(TRUE);

	PORTA = 0;
	PORTB = 0;
	PORTC = 0;
	PORTD = 0;
	PORTE = 0;
	RESETALL = 1;
	
	DECODER_EN  = 0;		// Multiplexer ausschalten (Gert 0)
	CHIPSEL     = 0;
	
	INTEDG2     = 1;		// steigende Flanke von IR
	INT2IF		= 0;
	
}


byte old,oldmin;
unsigned int16 dauer,wert;


////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////


void main()
{
	int16 l,m,status;
	byte k,p,i,merker;

	InitSystem();				// Globale Einstellungen
	Init_SPI(1);				// SPI einstellen
	AllDevicesOff();			// Alles abschalten
	SwitchDevice(devMMC,1);		// MMC muss anbleiben
	InitRTC();					// RTC starten
	RF_Init();					// RF Initialisieren
	RF_SetRX();					// RF auf Empfang
	RF_SetChannel(10);			// Kanal 0
	RF_Baudrate(b9600);			// Baudrate
	RF_SetRX_LNA(5,0,0);		// BW,LNA,RSSI setzen
	RF_RX_SetDQDFilter(3);

	//RF_TX_Power(2);				// Minimale Leistung

	// Timer IRQ klarmachen fr periodische Arbeiten

//	setup_timer_3(T3_INTERNAL | T3_DIV_BY_8);
//	enable_interrupts(INT_TIMER3);
	enable_interrupts(GLOBAL);

//	delay_ms(1500);				// Startbildschirm stehen lassen
	LCD_Clear();

	// ---------------------------------------------------------------------

	//TestDevices();

	merker = 0;
	while(1)
	{
		restart_wdt();
		enable_interrupts(GLOBAL);
		//RTC_ReadTimeDate();
		
		if (f_RF_RecInProgress && (!merker))
		{
			SwitchDevice(devLED_GLB,1);
			merker = 1;
		}

		if ((!f_RF_RecInProgress) && merker)
		{
			if (f_RF_RecError)
			{
				SwitchDevice(devLED_ROT,1);
				SwitchDevice(devLED_GRN,0);
			}
			else
			{
				SwitchDevice(devLED_GRN,1);
				SwitchDevice(devLED_ROT,0);
				SwitchDevice(devLED_GLB,0);
				merker = 0;
				sprintf(lcd,"%2x %2x %2x %2x %2x %2x",RF_RXBuf[0],RF_RXBuf[1],RF_RXBuf[2],RF_RXBuf[3],RF_RXBuf[4],RF_RXBuf[5]);
				LCD_Write(LCD,1);
				sprintf(lcd,"%2x %2x %2x %2x %2x %2x",RF_RXBuf[6],RF_RXBuf[7],RF_RXBuf[8],RF_RXBuf[9],RF_RXBuf[10],RF_RXBuf[11]);
				LCD_Write(LCD,2);
				sprintf(lcd,"N=%5lu Err=%5lu",Datensaetze,Errorcounter);
				LCD_Write(LCD,4);
				
			}

		}
	}
			
			
/*	
	while(1)
	{
		restart_wdt();
		enable_interrupts(GLOBAL);
		if (Zeit.sek!=old)
		{
			old=Zeit.sek;

			if (Zeit.min!=oldmin)
			{
				dauer++;
				oldmin=Zeit.min;	
			}	

			sprintf(LCDBuffer,"Zeit  : %02u:%02u:%02u",Zeit.std,Zeit.min,Zeit.sek);

			LCD_Write(LCDBuffer,1) ;
			sprintf(LCDBuffer,"Datum : %02u.%02u.%02u",Zeit.tag,Zeit.monat,Zeit.jahr);
			LCD_Write(LCDBuffer,2);
			
			if (Zeit.wtag==1)
			sprintf(LCDBuffer,"Tag   : Montag       ");
			if (Zeit.wtag==2)
			sprintf(LCDBuffer,"Tag   : Dienstag     ");
			if (Zeit.wtag==3)
			sprintf(LCDBuffer,"Tag   : Mittwoch     ");
			if (Zeit.wtag==4)
			sprintf(LCDBuffer,"Tag   : Donnerstag   ");
			if (Zeit.wtag==5)
			sprintf(LCDBuffer,"Tag   : Freitag      ");
			if (Zeit.wtag==6)
			sprintf(LCDBuffer,"Tag   : Samstag (WE) ");
			if (Zeit.wtag==7)
			sprintf(LCDBuffer,"Tag   : Sonntag (WE) ");
			LCD_Write(LCDBuffer,3);
			
			sprintf(LCDBuffer,"On    : %lu min",dauer);
			LCD_Write(LCDBuffer,4);
			
		}
		
	}	
*/
}

