/////////////////////////////////////////////////////////////////////////////
//
//			Ansteuerung des 16 Bit Shift Registers 74HCT595
//			zur Aktivierung diverser Gerte
//
/////////////////////////////////////////////////////////////////////////////


// >>> Informations-Flags ber Systemzustand (Read Only) <<<<<

bit	f_BusActivity;		// 1 = SPI Bus ist belegt




// Jedes Gert entspricht einem Bit am Shift Register, zB Traco = 0
enum devices {devTraco,devExtSupply,devSD_READ,devSD_WRITE,devTSOP,devPOTI,
			  devEE_LED,devRS232,devLED_ROT,devLED_GLB,devLED_GRN,devLED_BLAU,
			  dev_NUL1,dev_NUL2,dev_NUL3,devMMC};

// Front LEDs
enum LEDS {ROT,GELB,GRN,BLAU};

// Gerte am SPI Bus, Reihenfolge!
enum chips {devIR,devLCD,devRTC,devSD,devRAM,devRFM,devExt2,devNUL};

#define TAKTCYCLES			2	// Zyklen fr einen Impuls

int16 ShiftReg;				// Variable enthlt Bitmuster fr Gerte

///////////////////////////////////////////////////////////////////////////////////

// Schaltet alle Gerte aus
void AllDevicesOff()
{
	byte i;

	// Ports setzen, falls verstellt
	TRISD = TRISD & 0b11111100;		// D0,D1 = Out
	TRISC = TRISC & 0b11111101;		// C1 = Out
	
	SHIFTER_LOAD = 0;
	SHIFTER_CLK  = 0;
	
	ShiftReg = 0;					// alle aus

	// 16 Bits raustakten, MSB zuerst
	for (i=16;i>0;i--)
	{
		SHIFTER_DIN = bit_test(Shiftreg,i-1);		// Bit laden
		SHIFTER_CLK = 1;
		delay_cycles(1);
		SHIFTER_CLK = 0;
	}	
	
	// Latches aufmachen, Bitmuster erscheint an Ausgngen A0...A16
	SHIFTER_LOAD = 1;
	delay_cycles(TAKTCYCLES);
	SHIFTER_LOAD = 0;	

}	

///////////////////////////////////////////////////////////////////////////////////


// Gerte am 74595 ein (1) oder ausschalten (0)
void SwitchDevice(devices d,bit mode)
{
	byte i;

	disable_interrupts(GLOBAL);
	
	// Ports setzen, falls verstellt
	TRISD = TRISD & 0b11111100;		// D0,D1 = Out
	TRISC = TRISC & 0b11111101;		// C1 = Out

	SHIFTER_LOAD = 0;
	SHIFTER_CLK  = 0;

	// Neues Gert als Bit einfgen in globale Variable
	if (mode)
		 bit_set(ShiftReg,d);
	else
	     bit_clear(ShiftReg,d);	
	
	// 16 Bits raustakten, MSB zuerst
	for (i=16;i>0;i--)
	{
		SHIFTER_DIN = bit_test(Shiftreg,i-1);		// Bit laden
		SHIFTER_CLK = 1;
		delay_cycles(TAKTCYCLES);
		SHIFTER_CLK = 0;
	}	
	delay_cycles(2);
	
	// Latches aufmachen, Bitmuster erscheint an Ausgngen A0...A16
	SHIFTER_LOAD = 1;
	delay_cycles(TAKTCYCLES);
	SHIFTER_LOAD = 0;	

	enable_interrupts(GLOBAL);

}	

///////////////////////////////////////////////////////////////////////////////////

/*
	Schaltet die Front LEDs
	
	Eingabe: 0 		= ausschalten
			 1 		= einschalten
			 0xff	= keine Aktion
*/

void Switch_Front_LED(byte rot, gelb, grn, blau)
{
	byte i;
	
	// devLED_ROT,devLED_GLB,devLED_GRN,devLED_BLAU,
	
	// Die entsprechenden Bits setzen.....
	
	if (rot==1)
		bit_set(ShiftReg,devLED_ROT);
	if (rot==0)
		bit_clear(ShiftReg,devLED_ROT);	

	if (gelb==1)
		bit_set(ShiftReg,devLED_GLB);
	if (gelb==0)
		bit_clear(ShiftReg,devLED_GLB);	

	if (grn==1)
		bit_set(ShiftReg,devLED_GRN);
	if (grn==0)
		bit_clear(ShiftReg,devLED_GRN);	

	if (blau==1)
		bit_set(ShiftReg,devLED_BLAU);
	if (blau==0)
		bit_clear(ShiftReg,devLED_BLAU);	

	// 16 Bits raustakten, MSB zuerst
	for (i=16;i>0;i--)
	{
		SHIFTER_DIN = bit_test(Shiftreg,i-1);		// Bit laden
		SHIFTER_CLK = 1;
		delay_cycles(TAKTCYCLES);
		SHIFTER_CLK = 0;
	}
	
	// Latches aufmachen, Bitmuster erscheint an Ausgngen A0...A16
	SHIFTER_LOAD = 1;
	delay_us(1);
	SHIFTER_LOAD = 0;

}	


///////////////////////////////////////////////////////////////////////////////////


// Setzt die Chipselect Leitung eines am SPI Bus angeschlossenen Gertes
void ChipSelect(chips device,byte onoff)
{
	if (!onoff)
	{
		DECODER_EN = 0;
		delay_cycles(1);
		CHIPSEL = devNUL;
		f_BusActivity = FALSE;			// Passiven Bus anzeigen
	}
	else
	{
		CHIPSEL = device;
		delay_cycles(1);
		DECODER_EN = 1;					// Aktiven Bus anzeigen
		f_BusActivity = TRUE;
	}			
}	

///////////////////////////////////////////////////////////////////////////////////
// Konfiguriere die SPI als Master
// Speed = 0,1,2

void Init_SPI(byte speed)
{

 	TRISC = TRISC & 0b11010111;			// SDO,SCK auf Ausgang
 	TRISC = TRISC | 0b00010000;			// SDI Eingang

	// SPI einrichten

	switch (speed)
	{
		case 0:
				SSPCON1     = 0b00000010;		// Master Mode, fosc/64
				break;
		case 1:
				SSPCON1     = 0b00000001;		// Master Mode, fosc/16
				break;
		case 2:
				SSPCON1     = 0b00000000;		// Master Mode, fosc/4
				break;
	}

	SSPSTAT_SMP = 0;
	SSPCON1_CKP = 0;				// Daten bei steigender Flanke
	SSPSTAT_CKE = 0;
	SSPCON1_SSPEN = 1;				// SPI bleibt dauerhaft an
}


///////////////////////////////////////////////////////////////////////////////////
// SPI bedienen (Grne LED blinkt)

byte SPI(byte data, byte pause)
{
	byte rec;

    // Solange SPI beschftigt ist hmmere ihr die Daten ein bis frei wird   

	LED_AKTIV = 1;
	do {
        SSPCON1_WCOL = 0;
        SSPBUF = data;			// Sende Register beschreiben
    } while (SSPCON1_WCOL);

	// Datentransfer luft, jetzt warten bis Byte vom Slave reingekommen
    while (!SSPSTAT_BF);
	rec = SSPBUF;
	
    delay_us(PAUSE);    		// Notwendige Pause, damit Slave verarbeiten kann!
	LED_AKTIV = 0;

    return(rec);
}    


///////////////////////////////////////////////////////////////////////////////////
// Wartet solange bis Timeout oder erwartete Antwort kommt
//
// Eingabe: Sendebyte, erwartete Antwort, Maximale Zahl Versuche, Abstand zwischen Bytes (us)
// Rckgabe: 1=erfolgreich, 0=Timeout

byte SPI_WaitForAnswer(byte data,byte answer,int16 timeout,byte del)
{
    byte rec;

	LED_AKTIV = 1;
    while ((--timeout>0) && (rec!=answer))
    {
        restart_wdt();
        
        // Daten in die SPI einhmmern
        do {
            SSPCON1_WCOL = 0;
            SSPBUF = data;
        } while (SSPCON1_WCOL);

        // Sendung luft, wenn fertig Daten(mll) reinholen
        while (!SSPSTAT_BF);
	    rec = SSPBUF;

        // Definierte Pause, damit Slave reagieren kann
        delay_us(del);
    }
	LED_AKTIV = 0;
    
    if (rec==answer)
    	return(1);

   	return(0);
  
}    


///////////////////////////////////////////////////////////////////////////////////
// Sendet Anfrage an SPI Slave, ob empfangsbereit
//
// Chipselect bleibt gesetzt, wenn erfolgreich
//
// Ein:  SlaveNr, Zeitdauer in ms
// Rck: 1 = Slave antwortet

byte CheckForSlave(byte device, int16 to)
{
	byte flag;

	// Insgesamt 1 Sekunde lang versuchen Slave zu erreichen, dann abbrechen
	do {
		restart_wdt();
		ChipSelect(device,1);
		flag = SPI_WaitForAnswer(MASTER_READY,SLAVE_READY,100,50);	// ca 1ms
		if (!flag)
		{
			ChipSelect(device,0);
			delay_ms(1);
		}	
	} while ((!flag) && (--to>0));
	
	if (!to)
	{
		ChipSelect(device,0);
		return(0);
	}

	return(1);
}	


///////////////////////////////////////////////////////////////////////////////////
// Testet die Datenbertragung zu einem SPI Slave
// Rckgabe: 1 = Slave anwortet, 0 = Slave antwortet nicht

/*
	1. MASTER_READY		Aw: SLAVE_READY
	2. MASTER_START
	3. Master COMTEST	Aw: SLAVE ACK
	5. Master -> beliebige Daten an Slave, Sla ve antwortet invertiert
	
	Ende durch HIGH der SS Leitung
	
*/

byte Test_SPI_Device(byte device)
{
	#define TESTBYTES	512

	int16	foo;
	byte    k,answer;
	
	restart_wdt();
	Init_SPI(0);					// SPI konfigurieren

	// SPI Slave kontaktieren......
	if(!CheckForSlave(device,100))
	{
		foo=0;
		goto Test_SPI_Device_error;
	}
	
	// Header Infos senden
	SPI(MASTER_START,20);		// Master Startbyte senden
   	SPI(1,20);					// Datensatzgrsse senden
	SPI(MASTER_COMTEST,30);		// Comtest ankndigen


	// Besttigt Slave?	
 	if (!(SPI_WaitForAnswer(0xff,SLAVE_ACK,20,100)))
 	{
		foo=0;
		goto Test_SPI_Device_error;
	}
		
	// Daten senden und Empfang auswerten
	k = 0;
	SPI(k++,50);		// 1 Bytes in die Pipeline
	for (foo=0;foo<TESTBYTES;foo++)
	{
		answer = SPI(k,20);
		if ((k-1) != ~answer)		// Rckwert XOR'en
		{
			foo=0;
			goto Test_SPI_Device_error;
		}

		k++;
	
	}

	// ---- Raussprung ohne Fehler.... -------
	ChipSelect(device,0);
	return(1);
	
	// ----- Raussprung mit Fehler.... -------
	Test_SPI_Device_error:
	ChipSelect(device,0);
	return(0);
		
}	


