/*****************************************************************************/
/*###########################################################################*/
/* (c) Copyright by Varitronic AG 3177 Laupen Tel 031/747 60 60 Fax .. 60 62 */
/*###########################################################################*/
/*****************************************************************************/
/*                                                                           */
/* Hauptprogramm        : Def.h           			                         */
/* Kunde		: ADD AG 		                             */
/* Autor                : A.Husler			                                 */
/* Entwicklungsumgebung	: Dell PC, WINXP, CCS PCW C Compiler IDE             */
/* Beschreibung         : Prototypen Hauptprogramm, definitionen, fuses, uses*/
/* MCU                  : PIC24F Code   								 	 */
/* Derivat              : PIC24FJ16GA002                                     */
/* Letzte nderung		: 17.02.2010                                         */
/*                                                                           */
/*###########################################################################*/
/*****************************************************************************/
#ifndef _DEF_H
#define _DEF_H

#define DEBUG	OFF

/***************************************************************************/
/* * * * * * * * * * * * *   FUSE CONFIGURATION    * * * * * * * * * * * * */
/***************************************************************************/
/*
** Only one invocation of CONFIG1 should appear in a project,
** at the top of a C source file (outside of any function).
**
** The following constants can be used to set CONFIG1.
** Multiple options may be combined, as shown:
**
** _CONFIG1( OPT1_ON & OPT2_OFF & OPT3_PLL )
**
**   JTAG:
**     JTAGEN_OFF           Disabled
**     JTAGEN_ON            Enabled
**
**   Code Protect:
**     GCP_ON               Enabled
**     GCP_OFF              Disabled
**
**   Write Protect:
**     GWRP_ON              Enabled
**     GWRP_OFF             Disabled
**
**   Background Debugger:
**     BKBUG_ON             Enabled
**     BKBUG_OFF            Disabled
**
**   Clip-on Emulation mode:
**     COE_ON               Enabled
**     COE_OFF              Disabled
**
**   ICD pins select:
**     ICS_PGx3             EMUC/EMUD share PGC3/PGD3
**     ICS_PGx2             EMUC/EMUD share PGC2/PGD2
**     ICS_PGx1             EMUC/EMUD share PGC1/PGD1
**
**   Watchdog Timer:
**     FWDTEN_OFF           Disabled
**     FWDTEN_ON            Enabled
**
**   Windowed WDT:
**     WINDIS_ON            Enabled
**     WINDIS_OFF           Disabled
**
**   Watchdog prescaler:
**     FWPSA_PR32           1:32
**     FWPSA_PR128          1:128
**
**   Watchdog postscale:
**     WDTPS_PS1            1:1
**     WDTPS_PS2            1:2
**     WDTPS_PS4            1:4
**     WDTPS_PS8            1:8
**     WDTPS_PS16           1:16
**     WDTPS_PS32           1:32
**     WDTPS_PS64           1:64
**     WDTPS_PS128          1:128
**     WDTPS_PS256          1:256
**     WDTPS_PS512          1:512
**     WDTPS_PS1024         1:1,024
**     WDTPS_PS2048         1:2,048
**     WDTPS_PS4096         1:4,096
**     WDTPS_PS8192         1:8,192
**     WDTPS_PS16384        1:16,384
**     WDTPS_PS32768        1:32,768
**
// **************************************************************************3

** Only one invocation of CONFIG2 should appear in a project,
** at the top of a C source file (outside of any function).
**
** The following constants can be used to set CONFIG2.
** Multiple options may be combined, as shown:
**
** _CONFIG2( OPT1_ON & OPT2_OFF & OPT3_PLL )
**
**   Two Speed Start-up: Anschwingen mit dem schnellen internen Osci, wenn der externe Quarz
**   eingeschwungen ist, wird auf diesen Takt umgeschaltet
**     IESO_OFF             Disabled
**     IESO_ON              Enabled
**
**   Secondary Oscillator:
**     SOSCSEL_LPSOSC       Low Power Secondary Oscillator
**     SOSCSEL_SOSC         Default Secondary Oscillator
**
**   Voltage Regulator Standby-mode Wake-up Timer:
**     WUTSEL_FST           Fast Wake-up timer selected
**     WUTSEL_LEG           Legacy Wake-up timer selected
**
**   Oscillator Selection:
**     FNOSC_FRC            Fast RC oscillator
**     FNOSC_FRCPLL         Fast RC oscillator w/ divide and PLL
**     FNOSC_PRI            Primary oscillator (XT, HS, EC)
**     FNOSC_PRIPLL         Primary oscillator (XT, HS, EC) w/ PLL
**     FNOSC_SOSC           Secondary oscillator
**     FNOSC_LPRC           Low power RC oscillator
**     FNOSC_FRCDIV         Fast RC oscillator with divide
**
**   Clock switching and clock monitor:
**   Clock switching: Umschalten des Taktes per Software im Betrieb erlaubt
**   Clock monitor: Taktberwachung > Schaltet bei Strung auf Nottakt um
**     FCKSM_CSECME         Both enabled
**     FCKSM_CSECMD         Only clock switching enabled 
**     FCKSM_CSDCMD         Both disabled
**
**   OSCO/RC15 function:
**     OSCIOFNC_ON          RC15
**     OSCIOFNC_OFF         OSCO or Fosc/2
**
**   RP Register Protection:
**     IOL1WAY_OFF          Unlimited Writes To RP Registers
**     IOL1WAY_ON           Write RP Registers Once
**
**   I2C1 pins Select:
**     I2C1SEL_SEC          Use Secondary I2C1 pins
**     I2C1SEL_PRI          Use Primary I2C1 pins
**
**   Oscillator Selection:
**     POSCMOD_EC           External clock
**     POSCMOD_XT           XT oscillator
**     POSCMOD_HS           HS oscillator
**     POSCMOD_NONE         Primary disabled
**
*/

/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/* !!!!!!!!!!!!!!!!!!!!!! BEI JEDER NDERUNG ANPASSEN !!!!!!!!!!!!!!!!!!!!!! */
/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */

// Name und Version (werden im Eprom abgelegt)

#define SW_VERSION_Z                1
#define SW_VERSION_E                0
#define SW_VERSION                	10
#define SW_VERSION_INDEX            0
#define NAMESTRING                  "MMC-01  "
#define GERAETE_TYP					7	// Varitronic Gertenummer fr MMC-01 

#define FORMAT_ID                   10

/***************************************************************************/
/* * * * * * * * * * * * *      DEFINITIONEN     * * * * * * * * * * * * * */
/***************************************************************************/

/******************************************************************************/
/* * * * * * * * * * * *      SYSTEM KONSTANTEN       * * * * * * * * * * * * */
/******************************************************************************/

// Clear the bit 6 of OSCCONL to unlock Pin Re-map
#define PPSUnLock				__builtin_write_OSCCONL(OSCCON & 0xBF);
#define PPSLock					__builtin_write_OSCCONL(OSCCON | 0x40);//set the bit 6 of OSCCONL to lock Pin Re-map

// MCU
#define XTFREQ			8000000L         		// Int. RC Frequency / 2
//#define XTFREQ		7372800UL         		// On-board Crystal frequency
//#define PLLMODE       1               		// On-chip PLL setting bei FNOSC_FRC und POSCMOD_NONE
#define PLLMODE       	4               		// On-chip PLL setting bei FNOSC_PRIPLL und POSCMOD_HS oder
												// FNOSC_FRC und FNOSC_FRCPLL
#define FCY             (XTFREQ*PLLMODE)/2      // Instruction Cycle Frequency

// Timings ------------------------------------------------------------------- 

//#define ON_DELAY 			  700     	// 700ms Einschaltverzgerung
//#define OFF_DELAY 		  200     	// 200ms Ausschaltverzgerung
   	

/****************************************************************************/
/* * * * * * * * * * * *      PROJEKT KONSTANTEN    * * * * * * * * * * * * */
/****************************************************************************/

// KONSTANTS -------------------------------------------------------------------

#define OUTPUT	0
#define INPUT	1

#define ACTIV	0
#define INACTIV	1

#define OUT		0
#define IN		1

// PORT KONSTANTS --------------------------------------------------------------

#define nSEL_OUT_1 			_LATB2 = 1; // Select line for RFM12B Modul
#define nSEL_OUT_0 			_LATB2 = 0;	// Select line for RFM12B Modul
#define nSEL_TRIS			_TRISB2 	// Tris control for CS1_OUT pin

#define SCK_OUT_1 			_LATB3 = 1; // Select line for RFM12B Modul
#define SCK_OUT_0 			_LATB3 = 0;	// Select line for RFM12B Modul
#define SCK_TRIS			_TRISB3 	// Tris control for CS1_OUT pin

#define SDO_OUT_1 			_LATB6 = 1; // Select line for RFM12B Modul
#define SDO_OUT_0 			_LATB6 = 0;	// Select line for RFM12B Modul
#define SDO_TRIS			_TRISB6 	// Tris control for CS1_OUT pin

#define RFM12_SDI_IN 		_RB10 		// SDI from RFM12B Modul
#define RFM12_SDI_IN_TRIS	_TRISB10 	// Tris control for RFM12_SDI_IN pin

#define RFM12_DATA_IN 		_RA1 		// ASK Data from RFM12B Modul
#define RFM12_DATA_IN_TRIS	_TRISA1 	// Tris control for RFM12_DATA_IN pin

#define RFM12_nIRQ_IN 		_RB11 		// ASK Data from RFM12B Modul
#define RFM12_nIRQ_IN_TRIS	_TRISB11 	// Tris control for RFM12_DATA_IN pin

#define MAX3221_ENABLE_1	_LATB1 = 1;	
#define MAX3221_ENABLE_0	_LATB1 = 0;	

#define MAX3221_FORCEOFF_1	_LATB4 = 1;	
#define MAX3221_FORCEOFF_0	_LATB4 = 0;	

#define MAX3221_FORCEON_1	_LATB5 = 1;	
#define MAX3221_FORCEON_0	_LATB5 = 0;	

#define RECC_1				_LATB7 = 1;	
#define RECC_0				_LATB7 = 0;	
			
// #define LED1_EIN 	_LATB15 = 0;

#define LED1_EIN 	LATB &= ~0x8000;
#define LED1_AUS 	LATB |= 0x8000;

#define LED2_EIN 	LATB &= ~0x4000;
#define LED2_AUS 	LATB |= 0x4000;

#define LED3_EIN 	LATB &= ~0x2000;
#define LED3_AUS 	LATB |= 0x2000;

#define LED4_EIN 	LATB &= ~0x1000;
#define LED4_AUS 	LATB |= 0x1000;

/*
#define LED1_EIN 	PORTBbits.RB15 = 0;
#define LED1_AUS 	PORTBbits.RB15 = 1;

#define LED2_EIN 	PORTBbits.RB14 = 0;
#define LED2_AUS 	PORTBbits.RB14 = 1;

#define LED3_EIN 	PORTBbits.RB13 = 0;
#define LED3_AUS 	PORTBbits.RB13 = 1;

#define LED4_EIN 	PORTBbits.RB12 = 0;
#define LED4_AUS 	PORTBbits.RB12 = 1;
*/

#endif
